<?php
/**
 * 数据库管理类
 * 处理数据库连接和查询
 */

class DatabaseManager {
    private $conn;
    private $inTransaction = false;
    
    /**
     * 构造函数
     */
    public function __construct() {
        $this->connect();
    }
    
    /**
     * 连接数据库
     * 
     * @return bool 是否连接成功
     */
    private function connect() {
        try {
            $this->conn = new PDO(
                "mysql:host=" . DB_HOST . ";dbname=" . DB_NAME . ";charset=utf8mb4",
                DB_USER,
                DB_PASS,
                [
                    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
                    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
                    PDO::ATTR_EMULATE_PREPARES => false
                ]
            );
            return true;
        } catch (PDOException $e) {
            logMessage("数据库连接失败: " . $e->getMessage(), "ERROR");
            return false;
        }
    }
    
    /**
     * 执行查询
     * 
     * @param string $sql SQL语句
     * @param array $params 参数
     * @return PDOStatement|false 查询结果或失败
     */
    public function query($sql, $params = []) {
        try {
            $stmt = $this->conn->prepare($sql);
            $stmt->execute($params);
            return $stmt;
        } catch (PDOException $e) {
            logMessage("查询失败: " . $e->getMessage(), "ERROR");
            return false;
        }
    }
    
    /**
     * 获取单行数据
     * 
     * @param string $sql SQL语句
     * @param array $params 参数
     * @return array|false 数据或失败
     */
    public function getRow($sql, $params = []) {
        $stmt = $this->query($sql, $params);
        return $stmt ? $stmt->fetch() : false;
    }
    
    /**
     * 获取多行数据
     * 
     * @param string $sql SQL语句
     * @param array $params 参数
     * @return array|false 数据或失败
     */
    public function getRows($sql, $params = []) {
        $stmt = $this->query($sql, $params);
        return $stmt ? $stmt->fetchAll() : false;
    }
    
    /**
     * 获取单个值
     * 
     * @param string $sql SQL语句
     * @param array $params 参数
     * @return mixed|false 值或失败
     */
    public function getValue($sql, $params = []) {
        $stmt = $this->query($sql, $params);
        return $stmt ? $stmt->fetchColumn() : false;
    }
    
    /**
     * 插入数据
     * 
     * @param string $table 表名
     * @param array $data 数据
     * @return int|false 插入ID或失败
     */
    public function insert($table, $data) {
        $columns = implode(', ', array_keys($data));
        $placeholders = implode(', ', array_fill(0, count($data), '?'));
        
        $sql = "INSERT INTO {$table} ({$columns}) VALUES ({$placeholders})";
        
        $stmt = $this->query($sql, array_values($data));
        
        return $stmt ? $this->conn->lastInsertId() : false;
    }
    
    /**
     * 更新数据
     * 
     * @param string $table 表名
     * @param array $data 数据
     * @param string $where 条件
     * @param array $params 参数
     * @return int|false 影响行数或失败
     */
    public function update($table, $data, $where, $params = []) {
        $set = [];
        foreach ($data as $column => $value) {
            $set[] = "{$column} = ?";
        }
        
        $sql = "UPDATE {$table} SET " . implode(', ', $set) . " WHERE {$where}";
        
        $stmt = $this->query($sql, array_merge(array_values($data), $params));
        
        return $stmt ? $stmt->rowCount() : false;
    }
    
    /**
     * 删除数据
     * 
     * @param string $table 表名
     * @param string $where 条件
     * @param array $params 参数
     * @return int|false 影响行数或失败
     */
    public function delete($table, $where, $params = []) {
        $sql = "DELETE FROM {$table} WHERE {$where}";
        
        $stmt = $this->query($sql, $params);
        
        return $stmt ? $stmt->rowCount() : false;
    }
    
    /**
     * 开始事务
     * 
     * @return bool 是否成功
     */
    public function beginTransaction() {
        if ($this->inTransaction) {
            return true;
        }
        
        $this->inTransaction = $this->conn->beginTransaction();
        return $this->inTransaction;
    }
    
    /**
     * 提交事务
     * 
     * @return bool 是否成功
     */
    public function commit() {
        if (!$this->inTransaction) {
            return false;
        }
        
        $result = $this->conn->commit();
        $this->inTransaction = false;
        return $result;
    }
    
    /**
     * 回滚事务
     * 
     * @return bool 是否成功
     */
    public function rollback() {
        if (!$this->inTransaction) {
            return false;
        }
        
        $result = $this->conn->rollBack();
        $this->inTransaction = false;
        return $result;
    }
    
    /**
     * 关闭连接
     */
    public function close() {
        $this->conn = null;
    }
    
    /**
     * 析构函数
     */
    public function __destruct() {
        $this->close();
    }
}
