<?php
/**
 * 处理会员购买回调
 * 
 * @param int $chatId 聊天ID
 * @param int $userId 用户ID
 * @param string $callbackData 回调数据
 * @param object $db 数据库对象
 * @param object $bot TelegramBot对象
 * @return void
 */
function handleMembershipCallback($chatId, $userId, $callbackData, $db, $bot) {
    // 处理余额支付
    if (strpos($callbackData, 'pay_balance_') === 0) {
        $orderId = substr($callbackData, 12);
        $bot->handleBalancePayment($chatId, $userId, $orderId, $db);
        return;
    }
    
    // 处理检查支付状态
    if (strpos($callbackData, 'check_payment_') === 0) {
        $orderId = substr($callbackData, 14);
        checkPaymentStatus($chatId, $userId, $orderId, $db, $bot);
        return;
    }
    
    // 处理取消订单
    if (strpos($callbackData, 'cancel_order_') === 0) {
        $orderId = substr($callbackData, 13);
        cancelOrder($chatId, $userId, $orderId, $db, $bot);
        return;
    }
    
    // 处理充值
    if ($callbackData === 'recharge') {
        showRechargeOptions($chatId, $userId, $db, $bot);
        return;
    }
    
    // 处理赠送他人会员
    if ($callbackData === 'gift_membership') {
        // 提示用户输入要赠送的用户名
        $message = "请输入您要赠送会员的用户名\n\n用户名以 @ 开头，如 @ys_maidong\n请发送你需要开通 / 续费会员的Telegram用户名。";
        $bot->sendMessage($chatId, $message);
        
        // 更新用户状态为等待输入用户名
        saveUserState($chatId, 'waiting_gift_username', [], $db);
        return;
    }
    // 处理本账号开通/续费
    if ($callbackData === 'membership_self') {
        // 获取会员套餐列表并显示
        $plans = $db->getRows("SELECT * FROM membership_plans ORDER BY duration_days ASC");
        
        if (empty($plans)) {
            $bot->sendMessage($chatId, "暂无可用的会员套餐，请稍后再试。");
            return;
        }
        
        // 构建消息
        $message = "<b>请选择会员套餐</b>\n\n";
        
        foreach ($plans as $plan) {
            $message .= "{$plan['name']} / {$plan['price_usdt']} U\n";
        }
        
        // 构建按钮
        $buttons = [];
        
        foreach ($plans as $plan) {
            $buttons[] = [
                ['text' => "{$plan['name']} / {$plan['price_usdt']} U", 'callback_data' => "buy_membership_{$plan['plan_id']}"]
            ];
        }
        
        $buttons[] = [
            ['text' => '关闭', 'callback_data' => "close"]
        ];
        
        $keyboard = $bot->createInlineKeyboard($buttons);
        $bot->sendMessage($chatId, $message, $keyboard);
        return;
    }
    
    // 处理会员套餐选择
    if (strpos($callbackData, 'buy_membership_') === 0) {
        $planId = substr($callbackData, 15);
        
        // 获取会员套餐信息
        $plan = $db->getRow("SELECT * FROM membership_plans WHERE plan_id = ?", [$planId]);
        
        if (!$plan) {
            $bot->sendMessage($chatId, "会员套餐不存在，请重试。");
            return;
        }
        
        // 获取用户信息
        $user = $db->getRow("SELECT * FROM users WHERE id = ?", [$userId]);
        
        if (!$user) {
            $bot->sendMessage($chatId, "用户信息获取失败，请重试。");
            return;
        }
        
        // 处理会员购买
        $bot->handleMembershipPurchase($chatId, $userId, $user['balance'], $plan, $db);
        return;
    }
}

/**
 * 检查支付状态
 * 
 * @param int $chatId 聊天ID
 * @param int $userId 用户ID
 * @param int $orderId 订单ID
 * @param object $db 数据库对象
 * @param object $bot TelegramBot对象
 * @return void
 */
function checkPaymentStatus($chatId, $userId, $orderId, $db, $bot) {
    // 获取订单信息
    $order = $db->getRow("SELECT * FROM orders WHERE id = ? AND user_id = ?", [$orderId, $userId]);
    
    if (!$order) {
        $bot->sendMessage($chatId, "❌ 订单不存在或已过期，请重新下单。");
        return;
    }
    
    // 获取订单状态文本
    $statusText = getOrderStatusText($order['status']);
    
    // 构建消息
    $message = "<b>订单状态查询</b>\n\n";
    $message .= "订单号: {$order['order_number']}\n";
    $message .= "金额: {$order['amount']} USDT\n";
    $message .= "状态: {$statusText}\n";
    $message .= "创建时间: " . date('Y-m-d H:i:s', strtotime($order['created_at'])) . "\n";
    
    if ($order['status'] === 'pending') {
        // 计算剩余时间
        $expireTime = strtotime($order['expire_at']);
        $currentTime = time();
        $remainingTime = max(0, $expireTime - $currentTime);
        $remainingMinutes = floor($remainingTime / 60);
        $remainingSeconds = $remainingTime % 60;
        
        $message .= "过期时间: " . date('Y-m-d H:i:s', $expireTime) . "\n";
        $message .= "剩余时间: {$remainingMinutes}分{$remainingSeconds}秒\n\n";
        $message .= "请尽快完成支付，超时订单将自动取消。";
        
        // 构建按钮
        $buttons = [];
        
        // 获取用户信息
        $user = $db->getRow("SELECT * FROM users WHERE id = ?", [$userId]);
        
        // 检查用户余额是否足够
        if ($user && floatval($user['balance']) >= floatval($order['amount'])) {
            // 余额足够，显示使用余额支付按钮
            $buttons[] = [
                ['text' => '💰 使用余额支付', 'callback_data' => "pay_balance_{$orderId}"]
            ];
        } else if ($user) {
            // 余额不足，显示去充值按钮
            $message .= "\n\n您的余额: {$user['balance']} USDT (余额不足)";
            $buttons[] = [
                ['text' => '💰 去充值余额', 'callback_data' => "recharge"]
            ];
        }
        
        $buttons[] = [
            ['text' => '🔍 刷新状态', 'callback_data' => "check_payment_{$orderId}"]
        ];
        $buttons[] = [
            ['text' => '❌ 取消订单', 'callback_data' => "cancel_order_{$orderId}"]
        ];
        
        $keyboard = $bot->createInlineKeyboard($buttons);
        
        // 尝试编辑现有消息，如果失败则发送新消息
        $result = $bot->editMessageText($chatId, $order['message_id'] ?? null, $message, $keyboard);
        
        // 如果没有消息ID或编辑失败，则发送新消息并记录消息ID
        if (!isset($order['message_id']) || !$result['ok']) {
            $sentMessage = $bot->sendMessage($chatId, $message, $keyboard);
            if ($sentMessage['ok'] && isset($sentMessage['result']['message_id'])) {
                // 更新订单记录，保存消息ID
                $db->update('orders', [
                    'message_id' => $sentMessage['result']['message_id'],
                    'updated_at' => date('Y-m-d H:i:s')
                ], 'id = ?', [$orderId]);
            }
        }
    } else {
        // 订单已处理，显示状态
        if ($order['status'] === 'completed') {
            $message .= "\n✅ 您的订单已完成，感谢您的购买！";
        } else if ($order['status'] === 'cancelled') {
            $message .= "\n❌ 您的订单已取消。";
        } else if ($order['status'] === 'expired') {
            $message .= "\n⏱ 您的订单已过期，请重新下单。";
        } else if ($order['status'] === 'paid') {
            $message .= "\n💼 您的订单已支付，正在处理中...";
        }
        
        // 尝试编辑现有消息，如果失败则发送新消息
        if (isset($order['message_id'])) {
            $result = $bot->editMessageText($chatId, $order['message_id'], $message);
            if (!$result['ok']) {
                $bot->sendMessage($chatId, $message);
            }
        } else {
            $bot->sendMessage($chatId, $message);
        }
    }
}

/**
 * 取消订单
 * 
 * @param int $chatId 聊天ID
 * @param int $userId 用户ID
 * @param int $orderId 订单ID
 * @param object $db 数据库对象
 * @param object $bot TelegramBot对象
 * @return void
 */
function cancelOrder($chatId, $userId, $orderId, $db, $bot) {
    // 获取订单信息
    $order = $db->getRow("SELECT * FROM orders WHERE id = ? AND user_id = ? AND status = 'pending'", [$orderId, $userId]);
    
    if (!$order) {
        $bot->sendMessage($chatId, "❌ 订单不存在、已过期或已处理，无法取消。");
        return;
    }
    
    // 更新订单状态
    $db->update('orders', [
        'status' => 'cancelled',
        'updated_at' => date('Y-m-d H:i:s')
    ], 'id = ?', [$orderId]);
    
    // 发送确认消息
    $bot->sendMessage($chatId, "✅ 订单 {$order['order_number']} 已成功取消。");
}

/**
 * 显示充值选项
 * 
 * @param int $chatId 聊天ID
 * @param int $userId 用户ID
 * @param object $db 数据库对象
 * @param object $bot TelegramBot对象
 * @return void
 */
function showRechargeOptions($chatId, $userId, $db, $bot) {
    // 获取用户信息
    $user = $db->getRow("SELECT * FROM users WHERE id = ?", [$userId]);
    
    if (!$user) {
        $bot->sendMessage($chatId, "❌ 用户信息获取失败，请联系客服。");
        return;
    }
    
    // 构建消息
    $message = "<b>余额充值</b>\n\n";
    $message .= "当前余额: {$user['balance']} USDT\n\n";
    $message .= "请选择充值金额:";
    
    // 构建按钮
    $buttons = [
        [
            ['text' => '20 USDT', 'callback_data' => "recharge_amount_20"],
            ['text' => '50 USDT', 'callback_data' => "recharge_amount_50"]
        ],
        [
            ['text' => '100 USDT', 'callback_data' => "recharge_amount_100"],
            ['text' => '500 USDT', 'callback_data' => "recharge_amount_500"]
        ],
        [
            ['text' => '1000 USDT', 'callback_data' => "recharge_amount_1000"],
            ['text' => '5000 USDT', 'callback_data' => "recharge_amount_5000"]
        ],
        [
            ['text' => '❌ 关闭', 'callback_data' => "close_recharge"]
        ]
    ];
    
    $keyboard = $bot->createInlineKeyboard($buttons);
    $bot->sendMessage($chatId, $message, $keyboard);
}

// 注意：getOrderStatusText() 函数已移至bot.php，此处不再重复定义
