<?php
/**
 * 支付管理类
 * 处理支付流程和交易验证
 */

class PaymentManager {
    private $db;
    
    /**
     * 构造函数
     * 
     * @param DatabaseManager $db 数据库管理器
     */
    public function __construct($db) {
        $this->db = $db;
    }
    
    /**
     * 获取会员套餐信息
     * 
     * @param string $planId 套餐ID
     * @return array|false 套餐信息或失败
     */
    public function getPlanById($planId) {
        return $this->db->getRow("SELECT * FROM membership_plans WHERE plan_id = ?", [$planId]);
    }
    
    /**
     * 为指定套餐生成唯一价格
     * 在同一20分钟时间窗口内，同一套餐的不同订单会获得递增的价格（如16.01, 16.02等）
     * 
     * @param string $planId 套餐ID
     * @param float $basePrice 基础价格
     * @return float 唯一价格
     */
    public function getUniquePriceForPlan($planId, $basePrice) {
        try {
            // 获取过去20分钟内同一套餐的所有待支付订单的价格
            $timeWindow = date('Y-m-d H:i:s', strtotime('-' . ORDER_EXPIRATION_MINUTES . ' minutes'));
            $existingPrices = $this->db->getRows(
                "SELECT payment_amount FROM orders 
                 WHERE plan_id = ? AND status = 'pending' AND created_at > ?
                 ORDER BY payment_amount DESC",
                [$planId, $timeWindow]
            );
            
            if (empty($existingPrices)) {
                // 如果没有现有订单，使用基础价格加0.01
                return round($basePrice + 0.01, 2);
            }
            
            // 获取最高价格并加0.01
            $highestPrice = $existingPrices[0]['payment_amount'];
            // 确保价格至少比基础价格高0.01
            if ($highestPrice < $basePrice) {
                return round($basePrice + 0.01, 2);
            } else {
                return round($highestPrice + 0.01, 2);
            }
                
        } catch (Exception $e) {
            logMessage("获取唯一价格时发生数据库错误: " . $e->getMessage(), "ERROR");
            // 出错时返回基础价格加0.01，确保不会与原始价格冲突
            return round($basePrice + 0.01, 2);
        }
    }
    
    /**
     * 创建会员订单
     * 
     * @param int $userId 用户ID
     * @param string $planId 套餐ID
     * @param int $recipientId 接收者ID，如果是自己则与userId相同
     * @return array|false 订单信息或失败
     */
    public function createMembershipOrder($userId, $planId, $recipientId) {
        // 获取套餐信息
        $plan = $this->getPlanById($planId);
        
        if (!$plan) {
            logMessage("套餐不存在: " . $planId, "ERROR");
            return false;
        }
        
        // 检查用户是否已有相同套餐的待支付或活跃订单
        $existingOrder = $this->db->getRow(
            "SELECT id, status FROM orders 
             WHERE user_id = ? AND plan_id = ? AND status IN ('pending', 'paid', 'processing_gift', 'completed')
             ORDER BY created_at DESC LIMIT 1", 
            [$userId, $planId]
        );
        
        if ($existingOrder) {
            if ($existingOrder['status'] == 'pending') {
                return ['error' => "您已有一个待支付的 {$plan['name']} 订单 (订单号: {$existingOrder['id']})。请先完成支付或等待其过期。"];
            } else {
                return ['error' => "您当前已订阅 {$plan['name']} 或正在处理中。如需续费或有疑问，请联系管理员。"];
            }
        }
        
        // 获取唯一价格
        $basePrice = $plan['price_usdt'];
        $uniquePrice = $this->getUniquePriceForPlan($planId, $basePrice);
        
        // 生成唯一订单号
        $orderNumber = 'M' . time() . rand(1000, 9999);
        
        // 计算过期时间
        $expireAt = date('Y-m-d H:i:s', strtotime('+' . ORDER_EXPIRATION_MINUTES . ' minutes'));
        
        // 准备订单数据
        $orderData = [
            'order_number' => $orderNumber,
            'user_id' => $userId,
            'type' => 'membership',
            'plan_id' => $plan['plan_id'],
            'plan_name' => $plan['name'],
            'amount' => $basePrice,
            'status' => 'pending',
            'payment_address' => USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS,
            'payment_amount' => $uniquePrice,
            'recipient_telegram_id' => $recipientId,
            'membership_duration' => $plan['duration_days'],
            'expire_at' => $expireAt
        ];
        
        // 开始事务
        $this->db->beginTransaction();
        
        try {
            // 插入订单
            $orderId = $this->db->insert('orders', $orderData);
            
            if (!$orderId) {
                throw new Exception("创建订单失败");
            }
            
            // 创建交易记录
            $transactionData = [
                'user_id' => $userId,
                'order_id' => $orderId,
                'type' => 'membership',
                'amount' => $uniquePrice,
                'status' => 'pending'
            ];
            
            $transactionId = $this->db->insert('transactions', $transactionData);
            
            if (!$transactionId) {
                throw new Exception("创建交易记录失败");
            }
            
            // 提交事务
            $this->db->commit();
            
            // 返回订单信息
            return [
                'id' => $orderId,
                'order_number' => $orderNumber,
                'plan_name' => $plan['name'],
                'duration_days' => $plan['duration_days'],
                'base_price' => $basePrice,
                'payment_amount' => $uniquePrice,
                'payment_address' => USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS,
                'expire_at' => $expireAt
            ];
        } catch (Exception $e) {
            // 回滚事务
            $this->db->rollback();
            logMessage("创建订单失败: " . $e->getMessage(), "ERROR");
            return false;
        }
    }
    
    /**
     * 创建充值订单
     * 
     * @param int $userId 用户ID
     * @param float $amount 金额
     * @return array|false 订单信息或失败
     */
    public function createRechargeOrder($userId, $amount) {
        // 生成唯一订单号
        $orderNumber = 'R' . time() . rand(1000, 9999);
        
        // 计算过期时间
        $expireAt = date('Y-m-d H:i:s', strtotime('+' . ORDER_EXPIRATION_MINUTES . ' minutes'));
        
        // 准备订单数据
        $orderData = [
            'order_number' => $orderNumber,
            'user_id' => $userId,
            'type' => 'recharge',
            'amount' => $amount,
            'status' => 'pending',
            'payment_address' => USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS,
            'payment_amount' => $amount,
            'expire_at' => $expireAt
        ];
        
        // 开始事务
        $this->db->beginTransaction();
        
        try {
            // 插入订单
            $orderId = $this->db->insert('orders', $orderData);
            
            if (!$orderId) {
                throw new Exception("创建订单失败");
            }
            
            // 创建交易记录
            $transactionData = [
                'user_id' => $userId,
                'order_id' => $orderId,
                'type' => 'recharge',
                'amount' => $amount,
                'status' => 'pending'
            ];
            
            $transactionId = $this->db->insert('transactions', $transactionData);
            
            if (!$transactionId) {
                throw new Exception("创建交易记录失败");
            }
            
            // 提交事务
            $this->db->commit();
            
            // 返回订单信息
            return [
                'id' => $orderId,
                'order_number' => $orderNumber,
                'amount' => $amount,
                'payment_address' => USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS,
                'expire_at' => $expireAt
            ];
        } catch (Exception $e) {
            // 回滚事务
            $this->db->rollback();
            logMessage("创建订单失败: " . $e->getMessage(), "ERROR");
            return false;
        }
    }
    
    /**
     * 验证支付
     * 
     * @param string $txHash 交易哈希
     * @param string $orderNumber 订单号
     * @return bool 是否验证成功
     */
    public function verifyPayment($txHash, $orderNumber) {
        // 获取订单信息
        $order = $this->db->getRow("SELECT * FROM orders WHERE order_number = ?", [$orderNumber]);
        
        if (!$order) {
            logMessage("订单不存在: " . $orderNumber, "ERROR");
            return false;
        }
        
        // 检查订单状态
        if ($order['status'] != 'pending') {
            logMessage("订单状态不是待支付: " . $orderNumber, "ERROR");
            return false;
        }
        
        // 检查订单是否过期
        if (strtotime($order['expire_at']) < time()) {
            logMessage("订单已过期: " . $orderNumber, "ERROR");
            return false;
        }
        
        // 验证交易
        $transactionVerifier = new TransactionVerifier();
        $transaction = $transactionVerifier->verifyTransaction($txHash, $order['payment_amount'], $orderNumber);
        
        if (!$transaction) {
            logMessage("交易验证失败: " . $txHash, "ERROR");
            return false;
        }
        
        // 开始事务
        $this->db->beginTransaction();
        
        try {
            // 更新订单状态
            $this->db->update('orders', 
                ['status' => 'paid', 'payment_tx_hash' => $txHash, 'paid_at' => date('Y-m-d H:i:s')], 
                'id = ?', 
                [$order['id']]
            );
            
            // 更新交易状态
            $this->db->update('transactions', 
                ['status' => 'completed', 'tx_hash' => $txHash], 
                'order_id = ?', 
                [$order['id']]
            );
            
            // 处理订单
            if ($order['type'] == 'membership') {
                // 处理会员订单
                $this->processMembershipOrder($order);
            } else {
                // 处理充值订单
                $this->processRechargeOrder($order);
            }
            
            // 提交事务
            $this->db->commit();
            
            logMessage("支付验证成功: " . $orderNumber);
            return true;
        } catch (Exception $e) {
            // 回滚事务
            $this->db->rollback();
            logMessage("处理支付失败: " . $e->getMessage(), "ERROR");
            return false;
        }
    }
    
    /**
     * 处理会员订单
     * 
     * @param array $order 订单信息
     * @return bool 是否处理成功
     */
    private function processMembershipOrder($order) {
        // 获取用户信息
        $user = $this->db->getRow("SELECT * FROM users WHERE id = ?", [$order['user_id']]);
        
        if (!$user) {
            throw new Exception("用户不存在");
        }
        
        // 获取接收者ID
        $recipientId = $order['recipient_telegram_id'] ?: $user['telegram_id'];
        
        // 获取会员时长
        $duration = $order['membership_duration'];
        
        // 更新订单状态为处理中
        $this->db->update('orders', ['status' => 'processing_gift'], 'id = ?', [$order['id']]);
        
        // 激活会员
        $premiumActivator = new PremiumActivator($this->db);
        $result = $premiumActivator->activatePremium($recipientId, $duration);
        
        if (!$result) {
            throw new Exception("激活会员失败");
        }
        
        // 更新订单状态为完成
        $activatedAt = date('Y-m-d H:i:s');
        $this->db->update('orders', 
            [
                'status' => 'completed', 
                'activated_at' => $activatedAt
            ], 
            'id = ?', 
            [$order['id']]
        );
        
        // 更新用户订单数量
        $this->db->update('users', 
            ['order_count' => $user['order_count'] + 1], 
            'id = ?', 
            [$user['id']]
        );
        
        return true;
    }
    
    /**
     * 处理充值订单
     * 
     * @param array $order 订单信息
     * @return bool 是否处理成功
     */
    private function processRechargeOrder($order) {
        // 获取用户信息
        $user = $this->db->getRow("SELECT * FROM users WHERE id = ?", [$order['user_id']]);
        
        if (!$user) {
            throw new Exception("用户不存在");
        }
        
        // 更新用户余额
        $newBalance = $user['balance'] + $order['amount'];
        $this->db->update('users', 
            ['balance' => $newBalance, 'order_count' => $user['order_count'] + 1], 
            'id = ?', 
            [$user['id']]
        );
        
        // 更新订单状态为完成
        $this->db->update('orders', ['status' => 'completed'], 'id = ?', [$order['id']]);
        
        return true;
    }
    
    /**
     * 取消过期订单
     * 
     * @return int 取消的订单数量
     */
    public function cancelExpiredOrders() {
        $now = date('Y-m-d H:i:s');
        
        // 获取过期的待支付订单
        $expiredOrders = $this->db->getRows(
            "SELECT * FROM orders WHERE status = 'pending' AND expire_at < ?", 
            [$now]
        );
        
        $cancelCount = 0;
        
        foreach ($expiredOrders as $order) {
            // 更新订单状态为取消
            $result = $this->db->update('orders', 
                ['status' => 'cancelled'], 
                'id = ?', 
                [$order['id']]
            );
            
            if ($result) {
                // 更新交易状态为失败
                $this->db->update('transactions', 
                    ['status' => 'failed'], 
                    'order_id = ?', 
                    [$order['id']]
                );
                
                $cancelCount++;
                logMessage("订单已取消: " . $order['order_number']);
            }
        }
        
        return $cancelCount;
    }
}
