<?php
/**
 * Premium激活类
 * 处理Telegram Premium会员激活
 */

class PremiumActivator {
    private $db;
    private $botToken;
    
    /**
     * 构造函数
     * 
     * @param DatabaseManager $db 数据库管理器
     */
    public function __construct($db) {
        $this->db = $db;
        $this->botToken = BOT_TOKEN;
    }
    
    /**
     * 激活Premium会员
     * 
     * @param int $telegramId 用户Telegram ID
     * @param int $durationDays 会员时长（天）
     * @return bool 是否激活成功
     */
    public function activatePremium($telegramId, $durationDays) {
        // 将天数转换为月数
        $months = $this->convertDaysToMonths($durationDays);
        
        // 根据月数确定需要的星星数量
        $starCount = $this->getStarCountForMonths($months);
        
        // 调用Telegram API赠送Premium
        $result = $this->giftPremiumSubscription($telegramId, $months, $starCount);
        
        if ($result) {
            logMessage("成功为用户 {$telegramId} 开通了 {$months} 个月Premium会员");
            
            // 更新用户的Premium状态
            $this->updateUserPremiumStatus($telegramId, $durationDays);
            
            return true;
        } else {
            logMessage("为用户 {$telegramId} 开通Premium会员失败", "ERROR");
            return false;
        }
    }
    
    /**
     * 将天数转换为月数（3、6或12）
     * 
     * @param int $days 天数
     * @return int 月数
     */
    private function convertDaysToMonths($days) {
        if ($days <= 90) {
            return 3;
        } elseif ($days <= 180) {
            return 6;
        } else {
            return 12;
        }
    }
    
    /**
     * 根据月数获取需要的星星数量
     * 
     * @param int $months 月数
     * @return int 星星数量
     */
    private function getStarCountForMonths($months) {
        switch ($months) {
            case 3:
                return 1000;
            case 6:
                return 1500;
            case 12:
                return 2500;
            default:
                return 1000; // 默认为3个月
        }
    }
    
    /**
     * 调用Telegram API赠送Premium会员
     * 
     * @param int $userId 用户ID
     * @param int $monthCount 月数
     * @param int $starCount 星星数量
     * @return bool 是否成功
     */
    private function giftPremiumSubscription($userId, $monthCount, $starCount) {
        $url = "https://api.telegram.org/bot{$this->botToken}/giftPremiumSubscription";
        
        $data = [
            'user_id' => $userId,
            'month_count' => $monthCount,
            'star_count' => $starCount,
            'text' => "感谢您的购买！您的Telegram Premium会员已激活。"
        ];
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        
        if ($httpCode != 200) {
            logMessage("Telegram API请求失败: " . $httpCode . ", 响应: " . $response, "ERROR");
            return false;
        }
        
        $result = json_decode($response, true);
        
        if (!isset($result['ok']) || $result['ok'] !== true) {
            $errorMessage = $result['description'] ?? '未知错误';
            logMessage("Telegram API返回错误: " . $errorMessage, "ERROR");
            return false;
        }
        
        return true;
    }
    
    /**
     * 更新用户的Premium状态
     * 
     * @param int $telegramId 用户Telegram ID
     * @param int $durationDays 会员时长（天）
     * @return bool 是否更新成功
     */
    private function updateUserPremiumStatus($telegramId, $durationDays) {
        // 获取用户信息
        $user = $this->db->getRow("SELECT * FROM users WHERE telegram_id = ?", [$telegramId]);
        
        if (!$user) {
            logMessage("用户不存在: " . $telegramId, "ERROR");
            return false;
        }
        
        // 计算过期时间
        $expireDate = date('Y-m-d H:i:s', strtotime('+' . $durationDays . ' days'));
        
        // 更新用户Premium状态
        return $this->db->update('users', 
            [
                'is_premium' => 1, 
                'premium_expire_date' => $expireDate
            ], 
            'telegram_id = ?', 
            [$telegramId]
        ) !== false;
    }
}
