<?php
/**
 * Telegram机器人核心类
 * 处理会员购买流程，增加余额支付选项
 */

class TelegramBot {
    private $token;
    private $apiUrl;
    
    /**
     * 构造函数
     * 
     * @param string $token Bot Token
     */
    public function __construct($token) {
        $this->token = $token;
        $this->apiUrl = "https://api.telegram.org/bot{$token}/";
    }
    
    /**
     * 发送消息
     * 
     * @param int $chatId 聊天ID
     * @param string $text 消息文本
     * @param array $keyboard 键盘按钮
     * @return array|false 响应结果或失败
     */
    public function sendMessage($chatId, $text, $keyboard = null) {
        $data = [
            'chat_id' => $chatId,
            'text' => $text,
            'parse_mode' => 'HTML'
        ];
        
        if ($keyboard) {
            $data['reply_markup'] = json_encode($keyboard);
        }
        
        return $this->request('sendMessage', $data);
    }
    
    /**
     * 删除消息
     * 
     * @param int $chatId 聊天ID
     * @param int $messageId 消息ID
     * @return array|false 响应结果或失败
     */
    public function deleteMessage($chatId, $messageId) {
        $data = [
            'chat_id' => $chatId,
            'message_id' => $messageId
        ];
        
        return $this->request('deleteMessage', $data);
    }
    
    /**
     * 编辑消息文本
     * 
     * @param int $chatId 聊天ID
     * @param int $messageId 消息ID
     * @param string $text 新消息文本
     * @param array $keyboard 键盘按钮
     * @return array|false 响应结果或失败
     */
    public function editMessageText($chatId, $messageId, $text, $keyboard = null) {
        if (!$messageId) {
            return ['ok' => false, 'error' => 'No message ID provided'];
        }
        
        $data = [
            'chat_id' => $chatId,
            'message_id' => $messageId,
            'text' => $text,
            'parse_mode' => 'HTML'
        ];
        
        if ($keyboard) {
            $data['reply_markup'] = json_encode($keyboard);
        }
        
        return $this->request('editMessageText', $data);
    }
    
    /**
     * 回答回调查询
     * 
     * @param string $callbackQueryId 回调查询ID
     * @param string $text 通知文本
     * @param bool $showAlert 是否显示弹窗
     * @return array|false 响应结果或失败
     */
    public function answerCallbackQuery($callbackQueryId, $text = '', $showAlert = false) {
        $data = [
            'callback_query_id' => $callbackQueryId,
            'text' => $text,
            'show_alert' => $showAlert
        ];
        
        return $this->request('answerCallbackQuery', $data);
    }
    
    /**
     * 创建内联键盘
     * 
     * @param array $buttons 按钮数组
     * @return array 内联键盘数组
     */
    public function createInlineKeyboard($buttons) {
        return [
            'inline_keyboard' => $buttons
        ];
    }
    
    /**
     * 创建回复键盘
     * 
     * @param array $buttons 按钮数组
     * @param bool $resize 是否调整大小
     * @param bool $oneTime 是否一次性使用
     * @param bool $persistent 是否持久显示
     * @return array 回复键盘数组
     */
    public function createReplyKeyboard($buttons, $resize = true, $oneTime = false, $persistent = true) {
        return [
            'keyboard' => $buttons,
            'resize_keyboard' => $resize,
            'one_time_keyboard' => $oneTime,
            'persistent' => $persistent
        ];
    }
    
    /**
     * 处理会员购买
     * 
     * @param int $chatId 聊天ID
     * @param int $userId 用户ID
     * @param float $userBalance 用户余额
     * @param array $plan 会员套餐信息
     * @param object $db 数据库对象
     * @return void
     */
    public function handleMembershipPurchase($chatId, $userId, $userBalance, $plan, $db) {
        $planId = $plan['plan_id'];
        $duration = $plan['duration_days'];
        $price = $plan['price_usdt'];
        $planName = $plan['name'];
        
        // 生成唯一订单号
        $orderNumber = 'MEM' . time() . rand(1000, 9999);
        
        // 生成唯一支付金额，避免同时有多个相同金额的订单
        $paymentAmount = $this->generateUniqueAmount($price, $db);
        
        // 计算订单过期时间（20分钟后）
        $expireAt = date('Y-m-d H:i:s', strtotime('+20 minutes'));
        
        // 创建订单
        $orderId = $db->insert('orders', [
            'user_id' => $userId,
            'order_number' => $orderNumber,
            'type' => 'membership',
            'plan_id' => $planId,
            'amount' => $price,
            'plan_name' => $planName,
            'payment_amount' => $paymentAmount,
            'status' => 'pending',
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s'),
            'expire_at' => $expireAt
        ]);
        
        // 构建消息文本
        $message = "<b>正版会员购买</b>\n\n";
        $message .= "套餐: {$planName}\n";
        $message .= "时长: {$duration} 天\n";
        // $message .= "价格: {$price} USDT\n\n";
        $message .= "订单号: {$orderNumber}\n";
        $message .= "订单有效期: 20分钟\n\n";
        $message .= "<b>支付方式一:</b>\n";
        $message .= "请转账 <b>{$paymentAmount} USDT</b> 到以下地址:\n";
        $message .= "<code>" . USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS . "</code>\n";
        $message .= "网络: TRC20\n\n";
        $message .= "<b>支付方式二:</b>\n";
        $message .= "使用余额[{$userBalance}]支付\n\n";
        
        // 构建按钮
        $buttons = [];
        
        // 检查用户余额是否足够
        if ($userBalance >= $price) {
            // 余额足够，显示使用余额支付按钮
            $buttons[] = [
                ['text' => '💰 使用余额支付', 'callback_data' => "pay_balance_{$orderId}"]
            ];
        } else {
            // 余额不足，显示提示和去充值按钮
            $message .= "您的余额: {$userBalance} USDT (余额不足)\n\n";
            $buttons[] = [
                ['text' => '💰 去充值余额', 'callback_data' => "recharge"]
            ];
        }
        
        // 添加其他按钮
        $buttons[] = [
            ['text' => '🔍 检查支付状态', 'callback_data' => "check_payment_{$orderId}"]
        ];
        $buttons[] = [
            ['text' => '❌ 取消订单', 'callback_data' => "cancel_order_{$orderId}"]
        ];
        
        $keyboard = $this->createInlineKeyboard($buttons);
        
        // 发送消息
        $this->sendMessage($chatId, $message, $keyboard);
    }
    
    /**
     * 处理余额支付
     * 
     * @param int $chatId 聊天ID
     * @param int $userId 用户ID
     * @param int $orderId 订单ID
     * @param object $db 数据库对象
     * @return void
     */
    public function handleBalancePayment($chatId, $userId, $orderId, $db) {
        // 获取订单信息
        $order = $db->getRow("SELECT * FROM orders WHERE id = ? AND user_id = ?", [$orderId, $userId]);
        
        if (!$order) {
            $this->sendMessage($chatId, "❌ 订单不存在或已过期，请重新下单。");
            return;
        }
        
        if ($order['status'] !== 'pending') {
            $this->sendMessage($chatId, "❌ 该订单状态为 " . $this->getOrderStatusText($order['status']) . "，无法使用余额支付。");
            return;
        }
        
        // 获取用户信息
        $user = $db->getRow("SELECT * FROM users WHERE id = ?", [$userId]);
        
        if (!$user) {
            $this->sendMessage($chatId, "❌ 用户信息获取失败，请联系客服。");
            return;
        }
        
        // 检查余额是否足够
        if (floatval($user['balance']) < floatval($order['amount'])) {
            $buttons = [[['text' => '💰 去充值余额', 'callback_data' => "recharge"]]];
            $keyboard = $this->createInlineKeyboard($buttons);
            $this->sendMessage($chatId, "❌ 余额不足，无法完成支付。\n\n您的余额: {$user['balance']} USDT\n订单金额: {$order['amount']} USDT", $keyboard);
            return;
        }
        
        // 扣除余额
        $newBalance = floatval($user['balance']) - floatval($order['amount']);
        $db->update('users', [
            'balance' => $newBalance,
            'updated_at' => date('Y-m-d H:i:s')
        ], 'id = ?', [$userId]);
        
        // 更新订单状态
        $db->update('orders', [
            'status' => 'paid',
            'updated_at' => date('Y-m-d H:i:s')
        ], 'id = ?', [$orderId]);
        
        // 记录交易
        $db->insert('transactions', [
            'user_id' => $userId,
            'order_id' => $orderId,
            'amount' => $order['amount'],
            'type' => 'membership',
            'status' => 'completed',
            'description' => '余额支付会员',
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s')
        ]);
        
        // 处理会员开通
        if ($order['type'] == 'membership') {
            // 获取会员套餐信息
            $plan = $db->getRow("SELECT * FROM membership_plans WHERE plan_id = ?", [$order['plan_id']]);
            
            if ($plan) {
                // 激活Premium会员
                $duration = $plan['duration_days'];
                $premiumActivator = new PremiumActivator(BOT_TOKEN);
                $result = $premiumActivator->activatePremium($user['telegram_id'], $duration);
                
                if ($result) {
                    // 更新用户Premium状态
                    $premiumExpiry = date('Y-m-d H:i:s', strtotime("+{$duration} days"));
                    $db->update('users', [
                        'is_premium' => 1,
                        'premium_expiry' => $premiumExpiry,
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$userId]);
                    
                    // 更新订单状态为已完成
                    $db->update('orders', [
                        'status' => 'completed',
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$orderId]);
                    
                    // 发送成功消息
                    $message = "✅ <b>支付成功！</b>\n\n";
                    $message .= "您已成功使用余额支付并开通Premium会员\n";
                    $message .= "套餐: {$plan['name']}\n";
                    $message .= "时长: {$duration} 天\n";
                    $message .= "到期时间: " . date('Y-m-d H:i:s', strtotime($premiumExpiry)) . "\n\n";
                    $message .= "当前余额: {$newBalance} USDT";
                    
                    $this->sendMessage($chatId, $message);
                } else {
                    // 激活失败，退还余额
                    $db->update('users', [
                        'balance' => floatval($user['balance']),
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$userId]);
                    
                    // 更新订单和交易状态
                    $db->update('orders', [
                        'status' => 'failed',
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$orderId]);
                    
                    $db->update('transactions', [
                        'status' => 'failed',
                        'description' => '余额支付会员失败，已退款',
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'order_id = ?', [$orderId]);
                    
                    // 发送失败消息
                    $this->sendMessage($chatId, "❌ Premium会员开通失败，已退还余额。请联系客服处理。");
                }
            } else {
                $this->sendMessage($chatId, "❌ 会员套餐信息获取失败，请联系客服。");
            }
        }
    }
    
    /**
     * 生成唯一支付金额
     * 
     * @param float $baseAmount 基础金额
     * @param object $db 数据库对象
     * @return float 唯一支付金额
     */
    private function generateUniqueAmount($baseAmount, $db) {
        // 生成0.01-0.99之间的随机小数
        $randomCents = rand(1, 99) / 100;
        $amount = floatval($baseAmount) + $randomCents;
        $amount = round($amount, 2);
        
        // 检查是否已存在相同金额的待支付订单
        $existingOrder = $db->getRow("
            SELECT id FROM orders 
            WHERE payment_amount = ? AND status = 'pending' AND expire_at > NOW()
        ", [$amount]);
        
        // 如果存在，则递归重新生成
        if ($existingOrder) {
            return $this->generateUniqueAmount($baseAmount, $db);
        }
        
        return $amount;
    }
    
    /**
     * 获取订单状态文本
     * 
     * @param string $status 订单状态
     * @return string 状态文本
     */
    private function getOrderStatusText($status) {
        switch ($status) {
            case 'pending':
                return '待支付';
            case 'paid':
                return '已支付';
            case 'completed':
                return '已完成';
            case 'cancelled':
                return '已取消';
            case 'expired':
                return '已过期';
            case 'failed':
                return '失败';
            default:
                return '未知';
        }
    }
    
    /**
     * 发送API请求
     * 
     * @param string $method API方法
     * @param array $data 请求数据
     * @return array|false 响应结果或失败
     */
    private function request($method, $data) {
        $url = $this->apiUrl . $method;
        
        $options = [
            'http' => [
                'header' => "Content-Type: application/json\r\n",
                'method' => 'POST',
                'content' => json_encode($data),
                'ignore_errors' => true
            ]
        ];
        
        $context = stream_context_create($options);
        $response = file_get_contents($url, false, $context);
        
        if ($response === false) {
            return false;
        }
        
        return json_decode($response, true);
    }
}
