<?php
/**
 * Telegram机器人主菜单处理类
 * 实现底部按钮菜单功能
 */

class TelegramMenuHandler {
    private $botToken;
    private $db;
    
    /**
     * 构造函数
     * 
     * @param string $botToken Telegram Bot Token
     * @param DatabaseManager $db 数据库管理器
     */
    public function __construct($botToken, $db) {
        $this->botToken = $botToken;
        $this->db = $db;
    }
    
    /**
     * 发送API请求到Telegram
     * 
     * @param string $method API方法
     * @param array $params 参数
     * @return array 响应
     */
    private function sendRequest($method, $params = []) {
        $url = "https://api.telegram.org/bot{$this->botToken}/{$method}";
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($ch);
        curl_close($ch);
        
        return json_decode($response, true);
    }
    
    /**
     * 发送消息
     * 
     * @param int $chatId 聊天ID
     * @param string $text 消息文本
     * @param array $keyboard 键盘按钮
     * @return array 响应
     */
    public function sendMessage($chatId, $text, $keyboard = null) {
        $params = [
            'chat_id' => $chatId,
            'text' => $text,
            'parse_mode' => 'HTML'
        ];
        
        if ($keyboard) {
            $params['reply_markup'] = $keyboard;
        }
        
        return $this->sendRequest('sendMessage', $params);
    }
    
    /**
     * 创建底部主菜单按钮
     * 
     * @return array 键盘按钮
     */
    public function createMainMenu() {
        return [
            'keyboard' => [
                [
                    ['text' => '开会员'],
                    ['text' => '充值']
                ],
                [
                    ['text' => '客服'],
                    ['text' => '我的订单']
                ],
                [
                    ['text' => '个人中心'],
                    ['text' => '其他业务']
                ]
            ],
            'resize_keyboard' => true,
            'persistent' => true
        ];
    }
    
    /**
     * 处理开始命令
     * 
     * @param int $chatId 聊天ID
     * @param string $username 用户名
     * @param string $firstName 名字
     * @param string $lastName 姓氏
     * @return array 响应
     */
    public function handleStart($chatId, $username, $firstName, $lastName) {
        // 保存用户信息到数据库
        $this->saveUserInfo($chatId, $username, $firstName, $lastName);
        
        // 发送欢迎消息
        $welcomeText = "您好，欢迎使用Telegram会员充值机器人！\n\n请使用下方按钮选择服务：";
        
        return $this->sendMessage($chatId, $welcomeText, $this->createMainMenu());
    }
    
    /**
     * 处理主菜单按钮点击
     * 
     * @param int $chatId 聊天ID
     * @param string $text 按钮文本
     * @return array 响应
     */
    public function handleMainMenuClick($chatId, $text) {
        switch ($text) {
            case '开会员':
                return $this->handleMembership($chatId);
                
            case '充值':
                return $this->handleRecharge($chatId);
                
            case '客服':
                return $this->handleCustomerService($chatId);
                
            case '我的订单':
                return $this->handleMyOrders($chatId);
                
            case '个人中心':
                return $this->handleUserCenter($chatId);
                
            case '其他业务':
                return $this->handleOtherServices($chatId);
                
            default:
                return $this->sendMessage($chatId, "请使用菜单按钮选择服务", $this->createMainMenu());
        }
    }
    
    /**
     * 处理会员购买
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleMembership($chatId) {
        $text = "<b>正版会员</b>\n\n";
        $text .= "3个月 / " . MEMBERSHIP_3_MONTHS_PRICE . " U\n";
        $text .= "6个月 / " . MEMBERSHIP_6_MONTHS_PRICE . " U\n";
        $text .= "一年 / " . MEMBERSHIP_12_MONTHS_PRICE . " U\n";
        // $text .= "您要续费还是赠送会员？";
        
        $keyboard = [
            'inline_keyboard' => [
                [
                    ['text' => '会员开通/续费', 'callback_data' => 'membership_self']
                ],
                // [
                //     ['text' => '赠送他人会员', 'callback_data' => 'membership_gift']
                // ],
                [
                    ['text' => '关闭', 'callback_data' => 'close']
                ]
            ]
        ];
        
        return $this->sendMessage($chatId, $text, $keyboard);
    }
    
    /**
     * 处理充值
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleRecharge($chatId) {
        $text = "请选择充值金额：";
        
        $keyboard = [
            'inline_keyboard' => [
                [
                    ['text' => '20 USDT', 'callback_data' => 'recharge_amount_20'],
                    ['text' => '50 USDT', 'callback_data' => 'recharge_amount_50']
                ],
                [
                    ['text' => '100 USDT', 'callback_data' => 'recharge_amount_100'],
                    ['text' => '500 USDT', 'callback_data' => 'recharge_amount_500']
                ],
                [
                    ['text' => '1000 USDT', 'callback_data' => 'recharge_amount_1000'],
                    ['text' => '5000 USDT', 'callback_data' => 'recharge_amount_5000']
                ],
                [
                    ['text' => '关闭', 'callback_data' => 'close_recharge']
                ]
            ]
        ];
        
        return $this->sendMessage($chatId, $text, $keyboard);
    }
    
    /**
     * 处理客服
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleCustomerService($chatId) {
        $text = "如有疑问请联系客服 @" . CUSTOMER_SERVICE_USERNAME;
        
        return $this->sendMessage($chatId, $text, $this->createMainMenu());
    }
    
    /**
     * 处理我的订单
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleMyOrders($chatId) {
        // 获取用户ID
        $user = $this->getUserByTelegramId($chatId);
        
        if (!$user) {
            return $this->sendMessage($chatId, "用户信息不存在，请重新开始", $this->createMainMenu());
        }
        
        // 获取用户订单
        $orders = $this->db->getRows(
            "SELECT * FROM orders WHERE user_id = ? ORDER BY created_at DESC LIMIT 10",
            [$user['id']]
        );
        
        if (empty($orders)) {
            return $this->sendMessage($chatId, "您还没有订单记录", $this->createMainMenu());
        }
        
        $text = "<b>您的最近订单：</b>\n\n";
        
        foreach ($orders as $order) {
            $text .= "订单号：{$order['order_number']}\n";
            $text .= "类型：" . ($order['type'] == 'membership' ? '会员' : '充值') . "\n";
            $text .= "金额：{$order['amount']} USDT\n";
            $text .= "状态：" . $this->getOrderStatusText($order['status']) . "\n";
            $text .= "创建时间：" . date('Y-m-d H:i:s', strtotime($order['created_at'])) . "\n\n";
        }
        
        return $this->sendMessage($chatId, $text, $this->createMainMenu());
    }
    
    /**
     * 处理个人中心
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleUserCenter($chatId) {
        // 获取用户信息
        $user = $this->getUserByTelegramId($chatId);
        
        if (!$user) {
            return $this->sendMessage($chatId, "用户信息不存在，请重新开始", $this->createMainMenu());
        }
        
        $text = "<b>个人信息</b>\n";
        $text .= "=============\n";
        $text .= "ID：{$user['telegram_id']}\n";
        $text .= "用户名：" . ($user['username'] ? '@' . $user['username'] : '未设置') . "\n";
        $text .= "余额：{$user['balance']} U\n";
        $text .= "累计下单：{$user['order_count']} 单\n";
        
        if ($user['is_premium']) {
            $text .= "会员状态：已开通\n";
            $text .= "到期时间：" . date('Y-m-d', strtotime($user['premium_expire_date'])) . "\n";
        } else {
            $text .= "会员状态：未开通\n";
        }
        
        return $this->sendMessage($chatId, $text, $this->createMainMenu());
    }
    
    /**
     * 处理其他业务
     * 
     * @param int $chatId 聊天ID
     * @return array 响应
     */
    private function handleOtherServices($chatId) {
        return $this->sendMessage($chatId, "机器人开发【监听机器人,双向机器人,群发机器人,记账机器人,会员机器人,定制机器人等】,网站 app开发，联系客服 @ys_maidong", $this->createMainMenu());
    }
    
    /**
     * 保存用户信息
     * 
     * @param int $telegramId Telegram ID
     * @param string $username 用户名
     * @param string $firstName 名字
     * @param string $lastName 姓氏
     * @return bool 是否成功
     */
    private function saveUserInfo($telegramId, $username, $firstName, $lastName) {
        // 检查用户是否存在
        $user = $this->getUserByTelegramId($telegramId);
        
        if ($user) {
            // 更新用户信息
            return $this->db->update('users', 
                [
                    'username' => $username,
                    'first_name' => $firstName,
                    'last_name' => $lastName
                ], 
                'telegram_id = ?', 
                [$telegramId]
            ) !== false;
        } else {
            // 创建新用户
            return $this->db->insert('users', 
                [
                    'telegram_id' => $telegramId,
                    'username' => $username,
                    'first_name' => $firstName,
                    'last_name' => $lastName
                ]
            ) !== false;
        }
    }
    
    /**
     * 根据Telegram ID获取用户
     * 
     * @param int $telegramId Telegram ID
     * @return array|false 用户信息或失败
     */
    private function getUserByTelegramId($telegramId) {
        return $this->db->getRow("SELECT * FROM users WHERE telegram_id = ?", [$telegramId]);
    }
    
    /**
     * 获取订单状态文本
     * 
     * @param string $status 状态
     * @return string 状态文本
     */
    private function getOrderStatusText($status) {
        switch ($status) {
            case 'pending':
                return '待支付';
            case 'paid':
                return '已支付';
            case 'cancelled':
                return '已取消';
            case 'completed':
                return '已完成';
            case 'expired':
                return '已过期';
            case 'processing_gift':
                return '处理中';
            case 'failed_gifting':
                return '激活失败';
            default:
                return '未知状态';
        }
    }
}
