<?php
/**
 * 交易验证类
 * 验证USDT交易
 * 使用TronGrid API替代TronScan API
 */

class TransactionVerifier {
    private $walletAddress;
    private $contractAddress;
    private $apiKey;
    
    /**
     * 构造函数
     */
    public function __construct() {
        $this->walletAddress = USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS;
        $this->contractAddress = USDT_CONTRACT_ADDRESS;
        $this->apiKey = TRONGRID_API_KEY; // 从配置中获取API密钥
    }
    
    /**
     * 验证支付
     * 
     * @param float $expectedAmount 预期金额
     * @param string $orderNumber 订单号
     * @return array|false 交易信息或失败
     */
    public function verifyPayment($expectedAmount, $orderNumber) {
        // 记录开始验证
        logMessage("开始验证支付: 金额 {$expectedAmount} USDT, 订单号 {$orderNumber}");
        
        // 获取最近的交易记录
        $transactions = $this->getRecentTransactions();
        
        if (!$transactions || empty($transactions)) {
            logMessage("获取最近交易记录失败", "ERROR");
            return false;
        }
        
        // 遍历交易记录，查找匹配的交易
        foreach ($transactions as $transaction) {
            // 验证收款地址 - 严格匹配目标钱包
            if (!$this->isCorrectRecipient($transaction)) {
                continue;
            }
            
            // 验证金额 - 精确匹配预期金额
            if (!$this->isAmountCorrect($transaction, $expectedAmount)) {
                continue;
            }
            
            // 找到匹配的交易，只校验金额，不校验备注
            logMessage("找到匹配的交易: " . $transaction['transaction_id'] . ", 金额: " . ($transaction['value'] / 1000000) . " USDT");
            
            // 返回交易信息
            return [
                'txHash' => $transaction['transaction_id'],
                'amount' => $expectedAmount,
                'timestamp' => $transaction['block_timestamp'] / 1000, // 转换为秒
                'status' => 'confirmed'
            ];
        }
        
        // 未找到匹配的交易
        logMessage("未找到匹配的交易: 金额 {$expectedAmount} USDT, 订单号 {$orderNumber}");
        return false;
    }
    
    /**
     * 获取最近的交易记录
     * 使用TronGrid API替代TronScan API
     * 
     * @return array|false 交易记录或失败
     */
    private function getRecentTransactions() {
        // 使用TronGrid API获取最近的交易记录
        $url = "https://api.trongrid.io/v1/accounts/{$this->walletAddress}/transactions/trc20";
        
        $params = [
            "limit" => 100, // 增加限制，确保能获取到足够的交易
            "contract_address" => $this->contractAddress, // USDT合约地址
            "only_confirmed" => true
        ];
        
        $query = http_build_query($params);
        $url .= "?" . $query;
        
        logMessage("请求TronGrid API: " . $url);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // 允许重定向
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            "TRON-PRO-API-KEY: {$this->apiKey}", // 添加API密钥
            "Content-Type: application/json"
        ]);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curlError = curl_error($ch);
        curl_close($ch);
        
        // 记录详细的API响应信息
        logMessage("TronGrid API响应状态码: " . $httpCode);
        if (!empty($curlError)) {
            logMessage("cURL错误: " . $curlError, "ERROR");
        }
        
        if ($httpCode != 200) {
            logMessage("TronGrid API请求失败: " . $httpCode, "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return false;
        }
        
        $data = json_decode($response, true);
        
        // 检查API响应格式
        if (!isset($data['data']) || !is_array($data['data'])) {
            logMessage("API响应格式不符合预期，缺少data字段或格式错误", "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return false;
        }
        
        if (empty($data['data'])) {
            logMessage("交易数据为空");
            return false;
        }
        
        // 转换TronGrid API返回的数据格式
        // 只保留转入到目标钱包的交易
        $transactions = [];
        foreach ($data['data'] as $tx) {
            // 检查是否为TRC20转账
            if (isset($tx['type']) && $tx['type'] == 'Transfer') {
                // 检查是否为USDT转账
                if (isset($tx['token_info']['address']) && $tx['token_info']['address'] === $this->contractAddress) {
                    // 严格筛选：只处理转入到目标钱包的交易
                    if (isset($tx['to']) && $tx['to'] === $this->walletAddress) {
                        $transactions[] = [
                            'transaction_id' => $tx['transaction_id'],
                            'from' => $tx['from'],
                            'to' => $tx['to'],
                            'value' => $tx['value'], // 金额
                            'block_timestamp' => $tx['block_timestamp']
                        ];
                        
                        // 记录找到的转入交易
                        $amount = $tx['value'] / 1000000; // USDT有6位小数
                        logMessage("找到转入交易: " . $tx['transaction_id'] . ", 金额: " . $amount . " USDT");
                    }
                }
            }
        }
        
        logMessage("筛选后的交易数量: " . count($transactions));
        return $transactions;
    }
    
    /**
     * 验证交易
     * 
     * @param string $txHash 交易哈希
     * @param float $expectedAmount 预期金额
     * @param string $orderNumber 订单号
     * @return array|false 交易信息或失败
     */
    public function verifyTransaction($txHash, $expectedAmount, $orderNumber) {
        // 获取交易信息
        $transaction = $this->getTransactionInfo($txHash);
        
        if (!$transaction) {
            logMessage("获取交易信息失败: " . $txHash, "ERROR");
            return false;
        }
        
        // 验证收款地址 - 严格匹配目标钱包
        if (!$this->isCorrectRecipient($transaction)) {
            logMessage("收款地址不匹配: " . $txHash, "ERROR");
            return false;
        }
        
        // 验证金额 - 精确匹配预期金额
        if (!$this->isAmountCorrect($transaction, $expectedAmount)) {
            logMessage("金额不匹配: " . $txHash, "ERROR");
            return false;
        }
        
        // 不再验证备注，只验证金额
        return $transaction;
    }
    
    /**
     * 获取交易信息
     * 使用TronGrid API替代TronScan API
     * 
     * @param string $txHash 交易哈希
     * @return array|false 交易信息或失败
     */
    private function getTransactionInfo($txHash) {
        // 使用TronGrid API获取交易信息
        $url = "https://api.trongrid.io/v1/transactions/{$txHash}/events";
        
        logMessage("请求交易信息API: " . $url);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // 允许重定向
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            "TRON-PRO-API-KEY: {$this->apiKey}", // 添加API密钥
            "Content-Type: application/json"
        ]);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curlError = curl_error($ch);
        curl_close($ch);
        
        // 记录详细的API响应信息
        logMessage("交易信息API响应状态码: " . $httpCode);
        if (!empty($curlError)) {
            logMessage("cURL错误: " . $curlError, "ERROR");
        }
        
        if ($httpCode != 200) {
            logMessage("TronGrid API请求失败: " . $httpCode, "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return false;
        }
        
        $data = json_decode($response, true);
        
        if (empty($data) || !isset($data['data']) || empty($data['data'])) {
            logMessage("交易数据为空", "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return false;
        }
        
        // 查找USDT转账事件
        $usdtTransfer = null;
        foreach ($data['data'] as $event) {
            if (isset($event['contract_address']) && $event['contract_address'] === $this->contractAddress) {
                if (isset($event['event_name']) && $event['event_name'] === 'Transfer') {
                    // 找到USDT转账事件
                    $usdtTransfer = $event;
                    break;
                }
            }
        }
        
        if (!$usdtTransfer) {
            logMessage("未找到USDT转账事件", "ERROR");
            return false;
        }
        
        // 从事件中提取转账信息
        $from = null;
        $to = null;
        $value = null;
        
        if (isset($usdtTransfer['result'])) {
            foreach ($usdtTransfer['result'] as $param => $val) {
                if (strtolower($param) === 'from') {
                    $from = $val;
                } else if (strtolower($param) === 'to') {
                    $to = $val;
                } else if (strtolower($param) === 'value') {
                    $value = $val;
                }
            }
        }
        
        if (!$from || !$to || !$value) {
            logMessage("转账事件缺少必要参数", "ERROR");
            return false;
        }
        
        // 转换为统一格式
        return [
            'transaction_id' => $txHash,
            'from' => $from,
            'to' => $to,
            'value' => $value,
            'block_timestamp' => isset($usdtTransfer['block_timestamp']) ? $usdtTransfer['block_timestamp'] : time() * 1000
        ];
    }
    
    /**
     * 检查收款地址是否正确
     * 
     * @param array $transaction 交易信息
     * @return bool 是否正确
     */
    private function isCorrectRecipient($transaction) {
        return isset($transaction['to']) && $transaction['to'] === $this->walletAddress;
    }
    
    /**
     * 检查金额是否正确
     * 
     * @param array $transaction 交易信息
     * @param float $expectedAmount 预期金额
     * @return bool 是否正确
     */
    private function isAmountCorrect($transaction, $expectedAmount) {
        // 获取交易金额
        $amount = isset($transaction['value']) ? $transaction['value'] : 0;
        
        // TRC20 USDT的最小单位是6位小数
        $amount = $amount / 1000000;
        
        // 允许0.01 USDT的误差
        $isCorrect = abs($amount - $expectedAmount) <= 0.01;
        
        if ($isCorrect) {
            logMessage("金额匹配: 预期 {$expectedAmount} USDT, 实际 {$amount} USDT");
        } else {
            logMessage("金额不匹配: 预期 {$expectedAmount} USDT, 实际 {$amount} USDT");
        }
        
        return $isCorrect;
    }
}
