<?php
/**
 * USDT监控类
 * 监听TRC20钱包交易
 * 使用TronGrid API替代TronScan API
 */

class UsdtMonitor {
    private $db;
    private $walletAddress;
    private $contractAddress;
    private $paymentManager;
    private $apiKey;
    
    /**
     * 构造函数
     * 
     * @param DatabaseManager $db 数据库管理器
     * @param PaymentManager $paymentManager 支付管理器
     */
    public function __construct($db, $paymentManager) {
        $this->db = $db;
        $this->walletAddress = USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS;
        $this->contractAddress = USDT_CONTRACT_ADDRESS;
        $this->paymentManager = $paymentManager;
        $this->apiKey = TRONGRID_API_KEY; // 从配置中获取API密钥
    }
    
    /**
     * 监听交易
     * 
     * @return int 处理的交易数量
     */
    public function monitorTransactions() {
        // 获取最后检查的时间戳
        $lastCheckTime = $this->getLastCheckTime();
        
        // 获取待支付订单列表
        $pendingOrders = $this->getPendingOrders();
        
        if (empty($pendingOrders)) {
            logMessage("没有待支付订单");
            return 0;
        }
        
        logMessage("发现 " . count($pendingOrders) . " 笔待支付订单");
        
        // 获取新交易
        $transactions = $this->getNewTransactions($lastCheckTime);
        
        if (empty($transactions)) {
            logMessage("没有新交易");
            return 0;
        }
        
        logMessage("发现 " . count($transactions) . " 笔新交易");
        
        $processedCount = 0;
        
        // 处理每笔交易，匹配待支付订单
        foreach ($transactions as $transaction) {
            // 获取交易金额
            $value = $transaction['value'] ?? 0;
            $amount = $value / 1000000; // USDT有6位小数
            
            // 查找匹配的待支付订单
            $matchedOrder = null;
            foreach ($pendingOrders as $order) {
                // 精确匹配金额（允许0.01 USDT的误差）
                if (abs($amount - $order['payment_amount']) <= 0.01) {
                    $matchedOrder = $order;
                    break;
                }
            }
            
            if (!$matchedOrder) {
                logMessage("交易金额 {$amount} USDT 无匹配订单: " . $transaction['transaction_id']);
                continue;
            }
            
            // 处理匹配的交易
            if ($this->processTransaction($transaction, $matchedOrder)) {
                $processedCount++;
            }
        }
        
        // 更新最后检查时间
        $this->updateLastCheckTime();
        
        // 取消过期订单
        $cancelledCount = $this->paymentManager->cancelExpiredOrders();
        logMessage("已取消 " . $cancelledCount . " 笔过期订单");
        
        return $processedCount;
    }
    
    /**
     * 获取最后检查时间
     * 
     * @return int 时间戳
     */
    private function getLastCheckTime() {
        $lastCheck = $this->db->getValue("SELECT value FROM system_settings WHERE name = 'last_transaction_check'");
        
        if (!$lastCheck) {
            // 如果没有记录，默认为24小时前
            $lastCheck = time() - 86400;
            $this->db->query(
                "INSERT INTO system_settings (name, value) VALUES ('last_transaction_check', ?)",
                [$lastCheck]
            );
        }
        
        return intval($lastCheck);
    }
    
    /**
     * 更新最后检查时间
     * 
     * @return bool 是否成功
     */
    private function updateLastCheckTime() {
        $now = time();
        return $this->db->query(
            "UPDATE system_settings SET value = ? WHERE name = 'last_transaction_check'",
            [$now]
        ) !== false;
    }
    
    /**
     * 获取待支付订单列表
     * 
     * @return array 待支付订单列表
     */
    private function getPendingOrders() {
        return $this->db->getRows(
            "SELECT * FROM orders 
            WHERE status = 'pending' 
            AND expire_at > NOW() 
            ORDER BY created_at DESC"
        );
    }
    
    /**
     * 获取新交易
     * 使用TronGrid API替代TronScan API
     * 
     * @param int $lastCheckTime 最后检查时间
     * @return array 交易列表
     */
    private function getNewTransactions($lastCheckTime) {
        // 使用TronGrid API获取交易
        $url = "https://api.trongrid.io/v1/accounts/{$this->walletAddress}/transactions/trc20";
        
        $params = [
            "limit" => 100, // 增加限制，确保能获取到足够的交易
            "contract_address" => $this->contractAddress, // USDT合约地址
            "min_timestamp" => $lastCheckTime * 1000, // 转换为毫秒
            "only_confirmed" => true
        ];
        
        $query = http_build_query($params);
        $url .= "?" . $query;
        
        logMessage("请求TronGrid API: " . $url);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // 允许重定向
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            "TRON-PRO-API-KEY: {$this->apiKey}", // 添加API密钥
            "Content-Type: application/json"
        ]);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curlError = curl_error($ch);
        curl_close($ch);
        
        // 记录详细的API响应信息
        logMessage("TronGrid API响应状态码: " . $httpCode);
        if (!empty($curlError)) {
            logMessage("cURL错误: " . $curlError, "ERROR");
        }
        
        if ($httpCode != 200) {
            logMessage("TronGrid API请求失败: " . $httpCode, "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return [];
        }
        
        $data = json_decode($response, true);
        
        // 检查API响应格式
        if (!isset($data['data']) || !is_array($data['data'])) {
            logMessage("API响应格式不符合预期，缺少data字段或格式错误", "ERROR");
            logMessage("API响应内容: " . substr($response, 0, 500), "ERROR");
            return [];
        }
        
        if (empty($data['data'])) {
            logMessage("交易数据为空");
            return [];
        }
        
        // 筛选转入到目标钱包的交易
        $recentTransactions = [];
        
        foreach ($data['data'] as $tx) {
            // 检查是否为TRC20转账
            if (isset($tx['type']) && $tx['type'] == 'Transfer') {
                // 检查是否为USDT转账
                if (isset($tx['token_info']['address']) && $tx['token_info']['address'] === $this->contractAddress) {
                    // 严格筛选：只处理转入到目标钱包的交易
                    if (isset($tx['to']) && $tx['to'] === $this->walletAddress) {
                        $recentTransactions[] = [
                            'transaction_id' => $tx['transaction_id'],
                            'from' => $tx['from'],
                            'to' => $tx['to'],
                            'value' => $tx['value'], // TronGrid API返回的金额
                            'block_timestamp' => $tx['block_timestamp']
                        ];
                        
                        // 记录找到的转入交易
                        $amount = $tx['value'] / 1000000; // USDT有6位小数
                        logMessage("找到转入交易: " . $tx['transaction_id'] . ", 金额: " . $amount . " USDT");
                    }
                }
            }
        }
        
        logMessage("筛选后的交易数量: " . count($recentTransactions));
        return $recentTransactions;
    }
    
    /**
     * 处理交易
     * 
     * @param array $transaction 交易信息
     * @param array $order 匹配的订单
     * @return bool 是否处理成功
     */
    private function processTransaction($transaction, $order) {
        // 获取交易哈希
        $txHash = $transaction['transaction_id'] ?? '';
        
        if (empty($txHash)) {
            logMessage("交易哈希为空", "ERROR");
            return false;
        }
        
        // 检查交易是否已处理
        if ($this->isTransactionProcessed($txHash)) {
            logMessage("交易已处理: " . $txHash);
            return false;
        }
        
        // 获取交易金额
        $value = $transaction['value'] ?? 0;
        $amount = $value / 1000000; // USDT有6位小数
        
        logMessage("处理交易: " . $txHash . ", 金额: " . $amount . " USDT, 订单号: " . $order['order_number']);
        
        // 验证支付
        if ($this->paymentManager->verifyPayment($txHash, $order['order_number'])) {
            logMessage("支付验证成功: " . $order['order_number']);
            return true;
        } else {
            logMessage("支付验证失败: " . $order['order_number'], "ERROR");
            return false;
        }
    }
    
    /**
     * 检查交易是否已处理
     * 
     * @param string $txHash 交易哈希
     * @return bool 是否已处理
     */
    private function isTransactionProcessed($txHash) {
        return $this->db->getValue(
            "SELECT COUNT(*) FROM transactions WHERE tx_hash = ?",
            [$txHash]
        ) > 0;
    }
}
