<?php
/**
 * 监控脚本
 * 定时执行，监听USDT交易和处理过期订单
 */

// 引入配置文件
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/DatabaseManager.php';
require_once __DIR__ . '/TelegramBot.php';
require_once __DIR__ . '/TransactionVerifier.php';
require_once __DIR__ . '/PremiumActivator.php';

// 初始化日志函数
function logMessage($message, $level = 'INFO') {
    $logFile = __DIR__ . '/monitor.log';
    $timestamp = date('Y-m-d H:i:s');
    $logMessage = "[$timestamp] [$level] $message\n";
    file_put_contents($logFile, $logMessage, FILE_APPEND);
}

// 初始化数据库连接
$db = new DatabaseManager();

// 初始化Telegram机器人
$bot = new TelegramBot(BOT_TOKEN);

// 初始化交易验证器
$verifier = new TransactionVerifier(TRONGRID_API_KEY, USDT_USDT_USDT_USDT_USDT_WALLET_ADDRESS);

// 初始化Premium激活器
$activator = new PremiumActivator(BOT_TOKEN);

// 记录开始监控
logMessage("开始监控USDT交易...");
$startTime = microtime(true);

// 获取所有待支付订单
$pendingOrders = $db->getRows("
    SELECT o.*, u.telegram_id, u.username, u.first_name, u.last_name 
    FROM orders o 
    JOIN users u ON o.user_id = u.id 
    WHERE o.status = 'pending'
");

// 处理过期订单
$expiredOrders = $db->getRows("
    SELECT o.*, u.telegram_id, u.username, u.first_name, u.last_name 
    FROM orders o 
    JOIN users u ON o.user_id = u.id 
    WHERE o.status = 'pending' AND o.expire_at < NOW()
");

// 处理过期订单
$expiredCount = 0;
foreach ($expiredOrders as $order) {
    // 更新订单状态为已关闭
    $db->update('orders', [
        'status' => 'cancelled',
        'updated_at' => date('Y-m-d H:i:s')
    ], 'id = ?', [$order['id']]);
    
    // 发送通知消息给用户
    $message = "您的订单号[{$order['order_number']}]未及时支付，已被取消，请重新下单。";
    $bot->sendMessage($order['telegram_id'], $message);
    
    logMessage("订单 {$order['order_number']} 已过期并自动取消，已通知用户 {$order['telegram_id']}");
    $expiredCount++;
}

// 检查交易
$transactionCount = 0;
foreach ($pendingOrders as $order) {
    // 跳过已处理的过期订单
    if (in_array($order['id'], array_column($expiredOrders, 'id'))) {
        continue;
    }
    
    // 验证交易
    $transaction = $verifier->verifyPayment($order['payment_amount'], $order['order_number']);
    
    if ($transaction) {
        // 交易成功，更新订单状态
        $db->update('orders', [
            'status' => 'paid',
            'updated_at' => date('Y-m-d H:i:s')
        ], 'id = ?', [$order['id']]);
        
        // 记录交易
        $transactionId = $db->insert('transactions', [
            'user_id' => $order['user_id'],
            'order_id' => $order['id'],
            'tx_hash' => $transaction['txHash'],
            'amount' => $order['payment_amount'],
            'type' => $order['type'],
            'status' => 'completed',
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s')
        ]);
        
        // 处理不同类型的订单
        if ($order['type'] == 'membership') {
            // 获取会员套餐信息
            $plan = $db->getRow("SELECT * FROM membership_plans WHERE plan_id = ?", [$order['plan_id']]);
            
            if ($plan) {
                // 激活Premium会员
                $duration = $plan['duration_days'];
                $result = $activator->activatePremium($order['telegram_id'], $duration);
                
                if ($result) {
                    // 更新用户Premium状态
                    $premiumExpiry = date('Y-m-d H:i:s', strtotime("+{$duration} days"));
                    $db->update('users', [
                        'is_premium' => 1,
                        'premium_expiry' => $premiumExpiry,
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$order['user_id']]);
                    
                    // 更新订单状态为已完成
                    $db->update('orders', [
                        'status' => 'completed',
                        'updated_at' => date('Y-m-d H:i:s')
                    ], 'id = ?', [$order['id']]);
                    
                    // 发送通知消息给用户
                    $message = "恭喜！您的Premium会员已开通成功，有效期{$duration}天，到期时间：" . date('Y-m-d H:i:s', strtotime($premiumExpiry));
                    $bot->sendMessage($order['telegram_id'], $message);
                    
                    logMessage("用户 {$order['telegram_id']} 的Premium会员已激活，订单 {$order['order_number']} 已完成");
                } else {
                    logMessage("激活用户 {$order['telegram_id']} 的Premium会员失败", 'ERROR');
                    
                    // 发送通知消息给用户
                    $message = "您的Premium会员开通遇到问题，请联系客服处理。";
                    $bot->sendMessage($order['telegram_id'], $message);
                }
            }
        } elseif ($order['type'] == 'recharge') {
            // 充值订单，更新用户余额
            $user = $db->getRow("SELECT * FROM users WHERE id = ?", [$order['user_id']]);
            $newBalance = floatval($user['balance']) + floatval($order['amount']);
            
            $db->update('users', [
                'balance' => $newBalance,
                'updated_at' => date('Y-m-d H:i:s')
            ], 'id = ?', [$order['user_id']]);
            
            // 更新订单状态为已完成
            $db->update('orders', [
                'status' => 'completed',
                'updated_at' => date('Y-m-d H:i:s')
            ], 'id = ?', [$order['id']]);
            
            // 发送通知消息给用户
            $message = "充值成功！您已成功充值{$order['amount']} USDT，当前余额：{$newBalance} USDT";
            $bot->sendMessage($order['telegram_id'], $message);
            
            logMessage("用户 {$order['telegram_id']} 充值 {$order['amount']} USDT 成功，订单 {$order['order_number']} 已完成");
        }
        
        $transactionCount++;
    }
}

// 记录结束监控
$endTime = microtime(true);
$executionTime = round($endTime - $startTime, 2);
logMessage("监控完成，处理了 {$transactionCount} 笔交易，{$expiredCount} 个过期订单，耗时 {$executionTime} 秒");
